# This script creates a five-node topology(see the document) and
# three rsvp flows. All flows come from different nodes(2,3,4) and have the same 
# destination node(0). Also three trace files are created. Trace files are 
# created by logging the received bandwith in node zero for every flow. By  
# examining the trace files one can see the effect on received bandwith when   
# reservations are made. 
 
set ns [new Simulator] 
 
$ns color 0 red 
$ns color 1 blue 
$ns color 2 green 
$ns color 46 purple 
 
# Create nodes 
set n0 [$ns node] 
set n1 [$ns node] 
set n2 [$ns node] 
set n3 [$ns node] 
set n4 [$ns node] 
 
# Open trace files and activate nam-trace 
set f2 [open band02.tr w] 
set f3 [open band03.tr w] 
set f4 [open band04.tr w] 
set nf [open example1.nam w] 
$ns namtrace-all $nf 
 
# Create rsvp links(see the document!) 
$ns duplex-rsvp-link $n0 $n1 1Mb 1s 0.9 200 5000 Param Null 
$ns duplex-rsvp-link $n1 $n2 1Mb 1s 0.9 200 5000 Param Null 
$ns duplex-rsvp-link $n1 $n3 1Mb 1s 0.9 200 5000 Param Null 
$ns duplex-rsvp-link $n1 $n4 1Mb 1s 0.9 200 5000 Param Null 
 
 
 
# Enable upcalls on all nodes 
Agent/RSVP set noisy_ 255 
 
 
# Create rsvp agents 
set rsvp4 [$n4 add-rsvp-agent] 
set rsvp3 [$n3 add-rsvp-agent] 
set rsvp2 [$n2 add-rsvp-agent] 
set rsvp1 [$n1 add-rsvp-agent] 
set rsvp0 [$n0 add-rsvp-agent] 
 
 
 
# Create three traffic sources with desired bitrates. 
# Note that for some reason the random variable have to be turned on. 
# Note also that the value of "class_" have to be same for all flows(1). 
set udp2 [new Agent/UDP] 
$ns attach-agent $n2 $udp2 
$udp2 set class_ 1 
set cbr2 [new Application/Traffic/CBR] 
$cbr2 set rate_ 0.5Mb 
$cbr2 set random_ 1 
$cbr2 attach-agent $udp2 
 
set udp3 [new Agent/UDP] 
$ns attach-agent $n3 $udp3 
$udp3 set class_ 1 
set cbr3 [new Application/Traffic/CBR] 
$cbr3 set rate_ 0.5Mb 
$cbr3 set random_ 1 
$cbr3 attach-agent $udp3 
 
set udp4 [new Agent/UDP] 
$ns attach-agent $n4 $udp4 
$udp4 set class_ 1 
set cbr4 [new Application/Traffic/CBR] 
$cbr4 set rate_ 0.5Mb 
$cbr4 set random_ 1 
$cbr4 attach-agent $udp4 
 
 
# Create sinks 
set sink2 [new Agent/LossMonitor] 
$ns attach-agent $n0 $sink2 
 
set sink3 [new Agent/LossMonitor] 
$ns attach-agent $n0 $sink3 
 
set sink4 [new Agent/LossMonitor] 
$ns attach-agent $n0 $sink4 
 
 
# Connect traffic sources to sinks 
$ns connect $udp2 $sink2 
$ns connect $udp3 $sink3 
$ns connect $udp4 $sink4 
 
 
# Create rsvp sessions for all rsvp agents(see the document). Because the
# receiver nodes are same, the values of "class_" variable also have to be same(1).
set ses1 [$rsvp2 session $n0 1] 
set ses2 [$rsvp3 session $n0 1] 
set ses3 [$rsvp4 session $n0 1] 
 
 
 
# Start the procedure which logs the rate for the incoming flow on node 0 
$ns at 0.0 "record" 
 
 
# Send PATH-messages from each sender-node   
$ns at 1.0 "$rsvp2 sender $ses1 +500000 5000 32" 
$ns at 3.0 "$rsvp3 sender $ses2 +500000 5000 32" 
$ns at 5.0 "$rsvp4 sender $ses3 +500000 5000 32" 
 
 
# At 30.0 reserve 0,8Mb/s for flow coming from node three. The effect is seen
# in trace file. 
$ns at 30.0 "$rsvp0 reserve 0 ff +800000 100000 $n3" 
 
 
# Start and stop traffic sources 
$ns at 10.0 "$cbr2 start" 
$ns at 60.0 "$cbr2 stop" 
$ns at 10.0 "$cbr3 start" 
$ns at 60.0 "$cbr3 stop" 
$ns at 10.0 "$cbr4 start" 
$ns at 60.0 "$cbr4 stop" 
 
$ns at 70.0 "finish" 
 
 
 
# This procedure logs the received bandwiths for each flow. 
# For example the received bandwith from node two is logged 
# each second to trace file "band02.tr". 
proc record {} { 
	global sink2 sink3 sink4 ns f2 f3 f4 
	set time 1.0 
 
	set bw0 [$sink2 set bytes_] 
	set bw1 [$sink3 set bytes_] 
        set bw2 [$sink4 set bytes_] 
 
	set now [$ns now] 
 
	puts $f2 "$now [expr $bw0/$time*8]" 
	puts $f3 "$now [expr $bw1/$time*8]" 
	puts $f4 "$now [expr $bw2/$time*8]" 
 
	$sink2 set bytes_ 0 
	$sink3 set bytes_ 0 
	$sink4 set bytes_ 0 
 
	$ns at [expr $now+$time] "record" 
} 
 
 
proc finish {} { 
        global ns nf 
        $ns flush-trace 
        close $nf 
        exit 0 
} 
 
$ns run 

